import os
import subprocess
import sys
from pathlib import Path
import importlib.util

# 颜色常量
COLOR_RED = "\033[31m"
COLOR_GREEN = "\033[32m"
COLOR_YELLOW = "\033[33m"
COLOR_BLUE = "\033[34m"
COLOR_RESET = "\033[0m"

def find_project_root():
    """查找包含 manage.py 的项目根目录"""
    current_dir = Path.cwd()
    while True:
        if (current_dir / "manage.py").exists():
            return current_dir
        parent_dir = current_dir.parent
        if parent_dir == current_dir:
            print(f"{COLOR_RED}错误: 未找到 manage.py{COLOR_RESET}")
            sys.exit(1)
        current_dir = parent_dir

def get_project_module(project_root):
    """获取项目模块名称（同名文件夹）"""
    project_name = project_root.name
    module_dir = project_root / project_name
    if not (module_dir / "settings.py").exists():
        print(f"{COLOR_RED}错误: 未找到 {project_name}/settings.py{COLOR_RESET}")
        sys.exit(1)
    return project_name

def get_static_root(project_root):
    """从 settings 中获取 STATIC_ROOT"""
    project_module = get_project_module(project_root)
    sys.path.insert(0, str(project_root))
    for settings_module in [f"{project_module}.settings", f"{project_module}.local_settings"]:
        spec = importlib.util.find_spec(settings_module)
        if spec:
            settings = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(settings)
            if hasattr(settings, "STATIC_ROOT"):
                static_root = Path(getattr(settings, "STATIC_ROOT")).resolve().as_posix()
                return static_root if static_root.endswith('/') else f"{static_root}/"
    print(f"{COLOR_RED}错误: 未找到 STATIC_ROOT 配置{COLOR_RESET}")
    sys.exit(1)

def generate_nginx_config(port, static_root):
    """生成 Nginx 配置（移除 server_name 输入）"""
    return f"""
user nginx;
worker_processes auto;
error_log /var/log/nginx/error.log;
pid /run/nginx.pid;

include /usr/share/nginx/modules/*.conf;

events {{
    worker_connections 1024;
}}

http {{
    log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                      '$status $body_bytes_sent "$http_referer" '
                      '"$http_user_agent" "$http_x_forwarded_for"';

    access_log  /var/log/nginx/access.log  main;

    sendfile            on;
    tcp_nopush          on;
    tcp_nodelay         on;
    keepalive_timeout   65;
    types_hash_max_size 4096;

    include             /etc/nginx/mime.types;
    default_type        application/octet-stream;

    upstream django {{
        server 127.0.0.1:{port};
    }}

    server {{
        listen       80;
        listen       [::]:80;
        # 移除 server_name 配置，使用 Nginx 默认行为
        location /static/ {{
            alias   {static_root};
        }}

        location / {{
            uwsgi_pass  django;
            include     uwsgi_params;
        }}
    }}
}}
"""

def configure_nginx(port, static_root):
    """配置并重启 Nginx"""
    nginx_conf_path = "/etc/nginx/nginx.conf"
    backup_path = nginx_conf_path + ".bak"
    
    if Path(nginx_conf_path).exists():
        subprocess.run(f"cp {nginx_conf_path} {backup_path}", shell=True)
        print(f"{COLOR_GREEN}✔ 原配置已备份到 {backup_path}{COLOR_RESET}")
    
    config = generate_nginx_config(port, static_root)
    try:
        with open(nginx_conf_path, 'w') as f:
            f.write(config.strip())
        print(f"{COLOR_GREEN}✔ 新配置已写入 {nginx_conf_path}{COLOR_RESET}")
        
        if subprocess.run("nginx -t", shell=True, capture_output=True).returncode != 0:
            print(f"{COLOR_RED}错误: Nginx 配置测试失败{COLOR_RESET}")
            sys.exit(1)
        print(f"{COLOR_GREEN}✔ Nginx 配置测试通过{COLOR_RESET}")
        
        # 强制重启 Nginx 确保配置生效
        subprocess.run("systemctl restart nginx", shell=True, check=True)
        print(f"{COLOR_GREEN}✔ Nginx 服务已重启{COLOR_RESET}")
        
    except Exception as e:
        print(f"{COLOR_RED}错误: 配置失败 - {str(e)}{COLOR_RESET}")
        sys.exit(1)

def main():
    if os.getuid() != 0:
        print(f"{COLOR_RED}错误: 请使用 root 用户运行{COLOR_RESET}")
        sys.exit(1)

    print(f"{COLOR_BLUE}=== Django 项目 Nginx 部署工具 ==={COLOR_RESET}")
    
    project_root = find_project_root()
    print(f"{COLOR_GREEN}✔ 项目根目录: {project_root}{COLOR_RESET}")
    
    static_root = get_static_root(project_root)
    print(f"{COLOR_GREEN}✔ 静态文件目录: {static_root}{COLOR_RESET}")
    
    port = input(f"{COLOR_BLUE}请输入 Django 运行端口(默认 8000): {COLOR_RESET}") or "8000"
    
    print(f"{COLOR_BLUE}正在安装依赖...{COLOR_RESET}")
    try:
        subprocess.run("yum install -y nginx uwsgi uwsgi-plugin-python3", shell=True, check=True)
    except subprocess.CalledProcessError:
        print(f"{COLOR_RED}错误: 依赖安装失败{COLOR_RESET}")
        sys.exit(1)
    
    configure_nginx(port, static_root)
    
    print(f"\n{COLOR_GREEN}部署完成!{COLOR_RESET}")
    print(f"静态文件映射: /static/ => {static_root}")
    print(f"Django 运行端口: {port}")
    print("\n后续步骤:")
    print("1. 收集静态文件: python manage.py collectstatic --noinput")
    print("2. 启动 uWSGI: uwsgi --ini uwsgi.ini")

if __name__ == "__main__":
    main()